//                                               -*- C++ -*-
/**
 *  @file  Analytical.hxx
 *  @brief Analytical implements an algorithm to find the design point
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-06-26 13:50:17 +0200 (jeu, 26 jun 2008) $
 *  Id:      $Id: Analytical.hxx 862 2008-06-26 11:50:17Z dutka $
 */
#ifndef OPENTURNS_ANALYTICAL_HXX
#define OPENTURNS_ANALYTICAL_HXX

#include "Threadable.hxx"
#include "PersistentObject.hxx"
#include "NumericalPoint.hxx"
#include "NumericalPointWithDescription.hxx"
#include "NearestPointAlgorithm.hxx"
#include "NearestPointChecker.hxx"
#include "Collection.hxx"
#include "Event.hxx"
#include "StandardEvent.hxx"
#include "NumericalSample.hxx"
#include "NumericalMathFunction.hxx"
#include "Graph.hxx"
#include "Description.hxx"

namespace OpenTURNS
{

  namespace Uncertainty 
  {

    namespace Algorithm
    {

      /**
       * @class Analytical
       * NearestPointAlgorithm implements an algorithm to find the design point
       * and computes the results of an analytical result 
       */

      class Analytical
	: public Base::Common::PersistentObject,
	  public Base::Common::Threadable
      {

	CLASSNAME;
      public:

	typedef Base::Type::NumericalPoint                          NumericalPoint;
	typedef Base::Type::NumericalPointWithDescription           NumericalPointWithDescription;
	typedef Base::Type::Description                             Description;
	typedef Base::Optimisation::NearestPointAlgorithm           NearestPointAlgorithm;
	typedef Base::Func::NumericalMathFunction                   NumericalMathFunction;
	typedef Model::Distribution::NumericalPointCollection       NumericalPointCollection;
	typedef Model::Distribution::NumericalPointWithDescriptionCollection NumericalPointWithDescriptionCollection;
	typedef NumericalPointWithDescriptionCollection             Sensitivity;
	typedef Model::Event                                        Event;
	typedef Model::StandardEvent                                StandardEvent;
        typedef Base::Graph::Graph                                  Graph;
	typedef Graph::GraphCollection                              GraphCollection;

#ifndef SWIG
	/**
	 * @class Result
	 * Result stores the Analytical result 
	 */
	class Result
	  : public Base::Common::PersistentObject
	{
	  CLASSNAME;
	public:

	  static const NumericalScalar DefaultWidth = 1.0;

          typedef Analytical::Graph            Graph;
	  typedef Analytical::GraphCollection  GraphCollection;
	  typedef Base::Common::StorageManager StorageManager;

	  /** Standard constructor */
	  Result(const NumericalPoint & standardSpaceDesignPoint,
		 const Event & limitStateVariable,
		 const Bool isStandardPointOriginInFailureSpace,
                 const String & name = OT::DefaultName);

	  /* Default constructor (required by SWIG :-<) */
	  explicit Result();

	  /** Virtual constructor */
	  virtual Result * clone() const;

	  /** Destructor */
	  virtual ~Result();

	  /** Assigment operator */

	  /** StandardSpaceDesignPoint accessor */
	  NumericalPoint getStandardSpaceDesignPoint() const;

	  /** PhysicalSpaceDesignPoint accessor */
	  void setStandardSpaceDesignPoint(const NumericalPoint & standardSpaceDesignPoint);

	  /** PhysicalSpaceDesignPoint accessor */
	  NumericalPoint getPhysicalSpaceDesignPoint() const;

	  /** LimitStateVariable accessor */
	  Event getLimitStateVariable() const;

	  /** IsStandardPointOriginInFailureSpace  accessor */
	  Bool getIsStandardPointOriginInFailureSpace() const;

	  /** IsStandardPointOriginInFailureSpace  accessor */
	  void setIsStandardPointOriginInFailureSpace(const Bool isStandardPointOriginInFailureSpace);

	  /** ImportanceFactors accessor */
	  NumericalPoint getImportanceFactors();

	  /** ImportanceFactors graph */
	  Graph drawImportanceFactors();

	  /**  HasoferReliabilityIndex accessor */
	  NumericalScalar getHasoferReliabilityIndex() const;

	  /**  HasoferReliabilityIndexSensitivity accessor */
	  Sensitivity getHasoferReliabilityIndexSensitivity();

	  /** HasoferReliabilityIndexSensitivitygraph */
	  GraphCollection drawHasoferReliabilityIndexSensitivity(NumericalScalar width = DefaultWidth);

	  /** String converter */
	  String str() const;

	  /** Method save() stores the object through the StorageManager */
	  void save(const StorageManager::Advocate & adv) const;

	  /** Method load() reloads the object from the StorageManager */
	  void load(const StorageManager::Advocate & adv);

	protected:


	  /** Sensitivitygraph */
	  Graph drawSensitivity(const Sensitivity & sensitivity, NumericalScalar width) const;

	private:

	  /** PhysicalSpaceDesignPoint evaluation */
	  void computePhysicalSpaceDesignPoint();

	  /** HasoferReliabilityIndex evaluation */
	  void computeHasoferReliabilityIndex();

	  /** ImportanceFactors evaluation */
	  void computeImportanceFactors();

	  /** HasoferReliabilityIndexSensitivity  evaluation */
	  void computeHasoferReliabilityIndexSensitivity();

	  /** PhysicalSpaceDesignPoint accessor */
	  void setPhysicalSpaceDesignPoint(const NumericalPoint & physicalSpaceDesignPoint);

	  /** HasoferReliabilityIndex accessor */
	  void setHasoferReliabilityIndex(const NumericalScalar & hasoferReliabilityIndex);

	  /** ImportanceFactors accessor */
	  void setImportanceFactors(const NumericalPoint & importanceFactors);

	  /** Returns the position of the given (value, name) into the NumericalPoint or the dimension of the NumericalPoint if failed */
	  UnsignedLong computePosition(const String & marginalName, const String & marginalParameterName, const Description & parameterSetNames) const;

	  NumericalPoint    standardSpaceDesignPoint_;
	  NumericalPoint    physicalSpaceDesignPoint_;
	  Event             limitStateVariable_;
	  Bool isStandardPointOriginInFailureSpace_;
	  NumericalScalar hasoferReliabilityIndex_;
	  NumericalPointWithDescription importanceFactors_;
	  Sensitivity hasoferReliabilityIndexSensitivity_;
	  Bool isAlreadyComputedImportanceFactors_;
	  Bool isAlreadyComputedHasoferReliabilityIndexSensitivity_;

	}; // class Result
#else
	typedef OT::Uncertainty::Algorithm::AnalyticalResult Result;
#endif


	/** Constructor with parameters */
	explicit Analytical(const NearestPointAlgorithm & nearestPointAlgorithm,
			    const Event & compositeEvent,
			    const NumericalPoint & physicalStartingPoint);


	/** Virtual constructor */
	virtual Analytical * clone() const;

	/** Destructor */
	virtual ~Analytical();

	/** Physical starting point accessor */
	NumericalPoint getPhysicalStartingPoint() const;

	/** Physical starting point accessor */
	void setPhysicalStartingPoint(const NumericalPoint & physicalStartingPoint);

	/** Event accessor */
	Event getEvent() const;

	/** Event accessor */
	void setEvent(const Event & event);

	/** NearestPointAlgorithm accessor */
	NearestPointAlgorithm getNearestPointAlgorithm() const;

	/** NearestPointAlgorithm accessor */
	void setNearestPointAlgorithm(const NearestPointAlgorithm & nearestPointAlgorithm);

	/** Result accessor */
	virtual Result getAnalyticalResult() const;

	/** String converter */
	String str() const;

	/** Performs the actual computation. */
	void run();

      protected:

      private:

	NearestPointAlgorithm nearestPointAlgorithm_;
	Event event_;
	NumericalPoint physicalStartingPoint_;
	Result result_;

      } ; /* class Analytical */

      typedef OpenTURNS::Uncertainty::Algorithm::Analytical::Result AnalyticalResult;

    } /* namespace Algorithm */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_ANALYTICAL_HXX */
