//                                               -*- C++ -*-
/**
 *  @file  BoostPointerImplementation.hxx
 *  @brief The class BoostPointerImplementation implements a shared pointer 
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-05-10 16:43:31 +0200 (jeu, 10 mai 2007) $
 *  Id:      $Id: BoostPointerImplementation.hxx 434 2007-05-10 14:43:31Z dutka $
 */
#ifndef OPENTURNS_BOOSTPOINTERIMPLEMENTATION_HXX
#define OPENTURNS_BOOSTPOINTERIMPLEMENTATION_HXX

#include <boost/shared_ptr.hpp>   // for boost::shared_ptr
#include "OT.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Common
    {

      /**
       * @class BoostPointerImplementation
       *
       * BoostPointerImplementation implements a shared pointer strategy
       * based upon BOOST shared pointer
       */

      template <class T>
      class BoostPointerImplementation
      {
	/**
	 * Pointer_type gives access to the shared pointer
	 * implementation type
	 */
	typedef boost::shared_ptr<T> pointer_type;


	/**
	 * The actual shared pointer is \em ptr_
	 */
	pointer_type ptr_;

      public:

	/**
	 * Default constructor
	 * Constructed like this, the underlying pointer is NULL
	 */
	BoostPointerImplementation() : ptr_()
	{
	}

	/**
	 * Constructor from T * type
	 * The object pointed t is from now owned and taken in charge
	 * by the shared pointer. It MUST NOT be manually deleted
	 */
	explicit BoostPointerImplementation(T * ptr) : ptr_(ptr)
	{
	}

	/**
	 * Destructor does nothing
	 */
	~BoostPointerImplementation()
	{
	}

	/**
	 * Copy constructor adds one more reference
	 * on the underying object so its reference counter is
	 * incremented by one
	 */
	//// 	BoostPointerImplementation(const BoostPointerImplementation & ref) : ptr_(ref.ptr_)
	// 	{
	// 	}

	/**
	 * Copy constructor adds one more reference
	 * on the underying object so its reference counter is
	 * incremented by one
	 */
	template <class Derived>
	BoostPointerImplementation(const BoostPointerImplementation<Derived> & ref) : ptr_(ref.getImplementation())
	{
	}
#ifndef SWIG
	/**
	 * Equality operator adds one more reference
	 * on the underying object so its reference counter is
	 * incremented by one
	 */
	// 	BoostPointerImplementation & operator = (const BoostPointerImplementation & ref)
	// 	{
	// 	  ptr_ = ref.ptr_;
	// 	  return *this;
	// 	}
#endif

	/**
	 * Method to cast objects passed as base class into
	 * derived class
	 */
	template <class Base>
	BoostPointerImplementation & assign(const BoostPointerImplementation<Base> & ref)
	{
	  // We want to do : ptr_ = ref.ptr_
	  // but ref.ptr_ is a base class of ptr_
	  // so we need to dynamic cast it...

	  // Dynamic cast using a temporary
	  // 	  pointer_type tmp(ref.getImplementation(), boost::detail::dynamic_cast_tag());
	  // 	  ptr_ = tmp;

	  ptr_ = pointer_type(ref.getImplementation(), boost::detail::dynamic_cast_tag());
	  return *this;
	}



	/**
	 * Reset forsakes its reference on the pointed-to object.
	 * If the shared pointer is the only owner of the object,
	 * reset leads to its deletion
	 */
	inline void reset()
	{
	  ptr_.reset();
	}

	/**
	 * Reset forsakes its reference on the pointed-to object.
	 * If the shared pointer is the only owner of the object,
	 * reset leads to its deletion. This method takes the new
	 * pointed-to object and takes it in charge inside the
	 * shared pointer
	 */
	template <class Y>
	inline void reset(Y * p)
	{
	  ptr_.reset(p);
	}

	/**
	 * Operator * dereferences the const shared pointer and gives 
	 * access to the underlying object
	 */
	inline const T & operator * () const
	{
	  return *ptr_;
	}

	/**
	 * Operator -> dereferences the const shared pointer and gives 
	 * access to the underlying object
	 */
	inline const T * operator -> () const
	{
	  return ptr_.get();
	}

	/**
	 * Operator * dereferences the shared pointer and gives 
	 * access to the underlying object
	 */
	inline T & operator * ()
	{
	  return *ptr_;
	}

	/**
	 * Operator -> dereferences the shared pointer and gives 
	 * access to the underlying object
	 */
	inline T * operator -> ()
	{
	  return ptr_.get();
	}

	/**
	 * Get returns a pointer to the underlying object
	 */
	inline T * get() const
	{
	  return ptr_.get();
	}

	/**
	 * Method getImplementation() gives access to the 
	 * underlying implementation object (for copy
	 * constructor needs)
	 */
	inline const pointer_type & getImplementation() const
	{
	  return ptr_;
	}

	/**
	 * Unique returns true when the shared pointer is the only
	 * one that takes in charge the pointed-to object
	 * It is semantically equivalent to use_count() == 1
	 */
	inline OT::Bool unique() const
	{
	  return ptr_.unique();
	}

	/**
	 * Use_count returns the number of shared pointers that
	 * share the pointed-to object
	 */
	inline OT::UnsignedLong use_count() const
	{
	  return ptr_.use_count();
	}

	/**
	 * Swap exchanges the pointed-to objects between two
	 * shared pointers
	 */
	inline void swap(BoostPointerImplementation<T> & other)
	{
	  ptr_.swap(other.ptr_);
	}

      }

      ; /* class BoostPointerImplementation */


    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_BOOSTPOINTERIMPLEMENTATION_HXX */
