//                                               -*- C++ -*-
/**
 *  @file  CenteredFiniteDifferenceHessian.hxx
 *  @brief Class for the creation of a numerical math hessian implementation
 *         form a numerical math evaluation implementation by using centered
 *         finite difference formula.
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-05-24 12:02:17 +0200 (jeu, 24 mai 2007) $
 *  Id:      $Id: CenteredFiniteDifferenceHessian.hxx 446 2007-05-24 10:02:17Z dutka $
 */

#ifndef OPENTURNS_CENTEREDFINITEDIFFERENCEHESSIAN_HXX
#define OPENTURNS_CENTEREDFINITEDIFFERENCEHESSIAN_HXX

#define OPENTURNS_CENTEREDFINITEDIFFERENCEHESSIAN_HXX

#include "NumericalMathHessianImplementation.hxx"
#include "NumericalMathEvaluationImplementation.hxx"
#include "Exception.hxx"
#include "NumericalPoint.hxx"
#include "SymmetricTensor.hxx"
namespace OpenTURNS {

  namespace Base {

    namespace Diff {


      /**
       * @class CenteredFiniteDifferenceHessian
       *
       * This class is for the creation of a numerical math hessian implementation
       * form a numerical math evaluation implementation by using centered
       * finite difference formula
       */
      class CenteredFiniteDifferenceHessian
	: public Func::NumericalMathHessianImplementation
      {
	CLASSNAME;
      public:

	static const NumericalScalar DefaultEpsilon = 1e-4;

	typedef Func::NumericalMathHessianImplementation::Description              Description;
	typedef Func::NumericalMathHessianImplementation::SymmetricTensor          SymmetricTensor;
	typedef Func::NumericalMathHessianImplementation::NumericalPoint           NumericalPoint;
	typedef Func::NumericalMathHessianImplementation::InvalidArgumentException InvalidArgumentException;
	typedef Func::NumericalMathHessianImplementation::InternalException        InternalException;
	typedef Func::NumericalMathHessianImplementation::StorageManager           StorageManager;
	typedef Common::InvalidDimensionException                                  InvalidDimensionException;
	typedef Common::Pointer<Func::NumericalMathEvaluationImplementation>         EvaluationImplementation;

	/** Default constructor */
	CenteredFiniteDifferenceHessian(const String & name = OT::DefaultName);
	/** Parameter constructor */
	CenteredFiniteDifferenceHessian(const NumericalPoint & epsilon,
					const EvaluationImplementation & p_evaluation,
					const String & name = OT::DefaultName)
	  throw(InvalidDimensionException);

	/** Second parameter constructor */
	CenteredFiniteDifferenceHessian(const NumericalScalar epsilon,
					const EvaluationImplementation & p_evaluation,
					const String & name = OT::DefaultName)
	  throw(InvalidDimensionException);


	/** Virtual constructor */
	virtual CenteredFiniteDifferenceHessian * clone() const;

	/** Comparison operator */
	Bool operator ==(const CenteredFiniteDifferenceHessian & other) const;

	/** String converter */
	virtual String str() const;



	/* Here is the interface that all derived class must implement */
	
	/** Hessian () */
	virtual SymmetricTensor hessian(const NumericalPoint & in) const 
          throw(InvalidArgumentException, InternalException);

	/** Accessor for input point dimension */
	virtual UnsignedLong getInputNumericalPointDimension() const
          throw(InternalException);

	/** Accessor for output point dimension */
	virtual UnsignedLong getOutputNumericalPointDimension() const
	  throw(InternalException);

	/** Accessor for the epsilon */
	NumericalPoint getEpsilon() const;

	/** Accessor for the evaluation */
	EvaluationImplementation getEvaluation() const;

 	/** Method save() stores the object through the StorageManager */
	void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	void load(const StorageManager::Advocate & adv);

      protected:

    
      private:
	NumericalPoint epsilon_;
	EvaluationImplementation p_evaluation_;
      }; /* class CenteredFiniteDifferenceHessian */


    } /* namespace Func */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_CENTEREDFINITEDIFFERENCEHESSIAN_HXX */
