//                                               -*- C++ -*-
/**
 *  @file  Exponential.hxx
 *  @brief The Exponential distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-06-26 13:50:17 +0200 (jeu, 26 jun 2008) $
 *  Id:      $Id: Exponential.hxx 862 2008-06-26 11:50:17Z dutka $
 */
#ifndef OPENTURNS_EXPONENTIAL_HXX
#define OPENTURNS_EXPONENTIAL_HXX

#include "OT.hxx"
#include "NonEllipticalDistribution.hxx"
#include "Exception.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {


      /**
       * @class Exponential
       *
       * The Exponential distribution.
       */
      class Exponential 
	: public Model::NonEllipticalDistribution
      {
	CLASSNAME;
      public:

	typedef Base::Common::InvalidArgumentException               InvalidArgumentException;
	typedef Model::NonEllipticalDistribution                     NonEllipticalDistribution;  // required by SWIG
	typedef NonEllipticalDistribution::NumericalPoint            NumericalPoint;
	typedef NonEllipticalDistribution::NumericalSample           NumericalSample;
	typedef NonEllipticalDistribution::CovarianceMatrix          CovarianceMatrix;
	typedef NonEllipticalDistribution::NumericalPointWithDescriptionCollection  NumericalPointWithDescriptionCollection;
	typedef NonEllipticalDistribution::NotDefinedException       NotDefinedException;
	typedef NonEllipticalDistribution::StorageManager            StorageManager;


	/** Default constructor */
	Exponential();

	/** Parameters constructor */
	Exponential(const NumericalScalar lambda,
		    const NumericalScalar gamma)
	  throw(InvalidArgumentException);


	/** Comparison operator */
	Bool operator ==(const Exponential & other) const;

	/** String converter */
	String str() const;



	/* Interface inherited from Distribution */
	/** Virtual constructor */
	virtual Exponential * clone() const;	

	/** Get one realization of the distribution */
	virtual NumericalPoint getRealization() const;

	/** Get the DDF of the distribution */
	using NonEllipticalDistribution::computeDDF;
	virtual NumericalPoint computeDDF(const NumericalPoint & point) const;

	/** Get the PDF of the distribution */
	using NonEllipticalDistribution::computePDF;
	virtual NumericalScalar computePDF(const NumericalPoint & point) const;

	/** Get the CDF of the distribution */
	using NonEllipticalDistribution::computeCDF;
	virtual NumericalScalar computeCDF(const NumericalPoint & point) const;

	/** Get the characteristic function of the distribution, i.e. phi(u) = E(exp(I*u*X)) */
	NumericalComplex computeCharacteristicFunction(const NumericalPoint & point) const;

	/** Get the PDFGradient of the distribution */
	virtual NumericalPoint computePDFGradient(const NumericalPoint & point) const;

	/** Get the CDFGradient of the distribution */
	virtual NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

	/** Get the quantile of the distribution */
	virtual NumericalPoint computeQuantile(const NumericalScalar prob) const;

	/** Get the mean of the distribution */
	virtual NumericalPoint getMean() const throw(NotDefinedException);

	/** Get the standard deviation of the distribution */
	virtual NumericalPoint getStandardDeviation() const throw(NotDefinedException);

	/** Get the skewness of the distribution */
	virtual NumericalPoint getSkewness() const throw(NotDefinedException);

	/** Get the kurtosis of the distribution */
	virtual NumericalPoint getKurtosis() const throw(NotDefinedException);

	/** Get the covariance of the distribution */
	virtual CovarianceMatrix getCovariance() const throw(NotDefinedException);

	/** Parameters value and description accessor */
	virtual NumericalPointWithDescriptionCollection getParametersCollection() const;

	/* Interface specific to Exponential */

	/** Lambda accessor */
	void setLambda(const NumericalScalar lambda)
	  throw(InvalidArgumentException);
	NumericalScalar getLambda() const;

	/** Gamma accessor */
	void setGamma(const NumericalScalar gamma);
	NumericalScalar getGamma() const;

	/** Method save() stores the object through the StorageManager */
	virtual void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	virtual void load(const StorageManager::Advocate & adv);	

      protected:
    

      private:

	/** The lamdbda of the Exponential distribution */
	NumericalScalar lambda_;

	/** The gamma of the Exponential distribution */
	NumericalScalar gamma_;

      }; /* class Exponential */


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_EXPONENTIAL_HXX */
