//                                               -*- C++ -*-
/**
 *  @file  FORM.hxx
 *  @brief FORM implements the First Order Reliability Method
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-05-21 11:21:38 +0200 (mer, 21 mai 2008) $
 *  Id:      $Id: FORM.hxx 815 2008-05-21 09:21:38Z dutka $
 */
#ifndef OPENTURNS_FORM_HXX
#define OPENTURNS_FORM_HXX

#include "Analytical.hxx"

namespace OpenTURNS
{

  namespace Uncertainty
  {

    namespace Algorithm
    {

      /**
       * @class FORM
       * FORM implements the First Order Reliability Method
       * and compute the results of such kind of analyses
       */

      class FORM 
	: public Analytical
      {

	CLASSNAME;
      public:

	typedef Analytical::NumericalPoint        NumericalPoint;
	typedef Analytical::Event                 Event;
	typedef Analytical::Sensitivity           Sensitivity;
	typedef Analytical::NearestPointAlgorithm NearestPointAlgorithm;

#ifndef SWIG
	/**
	 * @class Result
	 * Result stores the FORM result 
	 */
	class Result:
	  public Analytical::Result
	{
	  CLASSNAME;
	public:

	  typedef Analytical::Result::Graph           Graph;
	  typedef Analytical::Result::GraphCollection GraphCollection;
          typedef Analytical::Result::StorageManager  StorageManager;

	  /** Standard constructor */
	  Result(const NumericalPoint & standardSpaceDesignPoint,
		 const Event & limitStateVariable,
		 const Bool isStandardPointOriginInFailureSpace,
                 const String & name = OT::DefaultName);

	  /* Default constructor (required by SWIG :-<) */
	  explicit Result();

	  /** Virtual constructor */
	  virtual Result * clone() const;

	  /** Destructor */
	  virtual ~Result();

	  /** Assigment operator */

	  /** EventProbability accessor */
	  NumericalScalar getEventProbability() const;

	  /** GeneralisedReliabilityIndex accessor */
	  NumericalScalar getGeneralisedReliabilityIndex() const;

	  /** EventProbabilitySensitivity accessor */
	  Sensitivity getEventProbabilitySensitivity();

	  /** HasoferReliabilityIndexSensitivitygraph */
	  GraphCollection drawEventProbabilitySensitivity(NumericalScalar width = DefaultWidth);

	  /** String converter */
	  String str() const;

	  /** Method save() stores the object through the StorageManager */
	  void save(const StorageManager::Advocate & adv) const;

	  /** Method load() reloads the object from the StorageManager */
	  void load(const StorageManager::Advocate & adv);

	private:

	  /** The function that actually evaluates the event probability with FORM approximation */
	  void computeEventProbability();

	  /** The function that actually evaluates the generalised reliability index with FORM approximation */
	  void computeGeneralisedReliabilityIndex();

	  /** The function that actually evaluates the  event probability sensitivity with FORM approximation */
	  void computeEventProbabilitySensitivity();

	  /** EventProbability accessor */
	  void setEventProbability(const NumericalScalar & eventProbability);

	  /** GeneralisedReliabilityIndex accessor */
	  void setGeneralisedReliabilityIndex(const NumericalScalar & generalisedReliabilityIndex);

	  /** EventProbabilitySensitivity accessor */
	  void setEventProbabilitySensitivity(const Sensitivity & eventProbabilitySensitivity);

	  NumericalScalar eventProbability_;
	  NumericalScalar generalisedReliabilityIndex_;
	  Sensitivity eventProbabilitySensitivity_;
	  Bool isAlreadyComputedEventProbabilitySensitivity_;

	}; // class Result
#else
	typedef OT::Uncertainty::Algorithm::FORMResult Result;
#endif

	/** Constructor with parameters */
	explicit FORM(const NearestPointAlgorithm & nearestPointAlgorithm,
		      const Event & event,
		      const NumericalPoint & physicalStartingPoint);


	/** Virtual constructor */
	virtual FORM * clone() const;

	/** Destructor */
	virtual ~FORM();

	/** Result accessor */
	Result getResult() const;

	/** Result accessor */
	void setResult(const Result & result);

	/** String converter */
	String str() const;

	/** Function that computes the design point by re-using the Analytical::run() and creates a FORM::Result */
	void run();

      private:

	Result result_;
      } ; /* class FORM */

      typedef OpenTURNS::Uncertainty::Algorithm::FORM::Result FORMResult;

    } /* namespace Algorithm */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_FORM_HXX */



