//                                               -*- C++ -*-
/**
 *  @file  FrankCopula.hxx
 *  @brief The FrankCopula distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-06-26 13:50:17 +0200 (jeu, 26 jun 2008) $
 *  Id:      $Id: FrankCopula.hxx 862 2008-06-26 11:50:17Z dutka $
 */
#ifndef OPENTURNS_FRANKCOPULA_HXX
#define OPENTURNS_FRANKCOPULA_HXX

#include "ArchimedeanCopula.hxx"
#include "Exception.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {


      /**
       * @class FrankCopula
       *
       * The FrankCopula distribution.
       */
      class FrankCopula 
	: public Model::ArchimedeanCopula
      {
	CLASSNAME;
      public:

	typedef Base::Common::InvalidArgumentException       InvalidArgumentException;
	typedef Model::ArchimedeanCopula                     ArchimedeanCopula;  // required by SWIG
	typedef ArchimedeanCopula::NumericalPoint            NumericalPoint;
	typedef ArchimedeanCopula::NumericalPointWithDescriptionCollection  NumericalPointWithDescriptionCollection;
	typedef ArchimedeanCopula::NotDefinedException       NotDefinedException;
	typedef ArchimedeanCopula::StorageManager            StorageManager;


	/** Default constructor */
	FrankCopula();

	/** Parameters constructor */
	FrankCopula(const NumericalScalar theta)
	  throw(InvalidArgumentException);


	/** Comparison operator */
	Bool operator ==(const FrankCopula & other) const;

	/** String converter */
	String str() const;



	/* Interface inherited from Distribution */
	/** Virtual constructor */
	virtual FrankCopula * clone() const;	

	/** Get one realization of the distribution */
	virtual NumericalPoint getRealization() const;

	/** Get the DDF of the distribution */
	using ArchimedeanCopula::computeDDF;
	virtual NumericalPoint computeDDF(const NumericalPoint & point) const;

	/** Get the PDF of the distribution */
	using ArchimedeanCopula::computePDF;
	virtual NumericalScalar computePDF(const NumericalPoint & point) const;

	/** Get the CDF of the distribution */
	using ArchimedeanCopula::computeCDF;
	virtual NumericalScalar computeCDF(const NumericalPoint & point) const;

	/** Get the PDFGradient of the distribution */
	virtual NumericalPoint computePDFGradient(const NumericalPoint & point) const;

	/** Get the CDFGradient of the distribution */
	virtual NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

	/** Get the quantile of the distribution */
	virtual NumericalPoint computeQuantile(const NumericalScalar prob) const;

	/** Compute the archimedean generator of the archimedean copula, i.e.
	 *  the function phi such that the CDF of the copula can
	 *  be written as CDF(t) = phi^{-1}(phi(u)+phi(v))
	 */
	virtual NumericalScalar computeArchimedeanGenerator(const NumericalScalar t) const;

	/** Compute the inverse of the archimedean generator */
	virtual NumericalScalar computeInverseArchimedeanGenerator(const NumericalScalar t) const;

	/** Compute the derivative of the archimedean generator */
	virtual NumericalScalar computeArchimedeanGeneratorDerivative(const NumericalScalar t) const;

	/** Compute the second derivative of the archimedean generator */
	virtual NumericalScalar computeArchimedeanGeneratorSecondDerivative(const NumericalScalar t) const;

	/** Parameters value and description accessor */
	virtual NumericalPointWithDescriptionCollection getParametersCollection() const;

	/** Tell if the distribution has independent copula */
	virtual Bool hasIndependentCopula() const;

	/* Interface specific to FrankCopula */

	/** Theta accessor */
	void setTheta(const NumericalScalar theta)
	  throw(InvalidArgumentException);
	NumericalScalar getTheta() const;

	/** Method save() stores the object through the StorageManager */
	virtual void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	virtual void load(const StorageManager::Advocate & adv);	

      protected:
    

      private:

	/** The parameter of the FrankCopula distribution */
	NumericalScalar theta_;

      }; /* class FrankCopula */


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_FRANKCOPULA_HXX */
