//                                               -*- C++ -*-
/**
 *  @file  Log.hxx
 *  @brief Log records all user information to a file or tty
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-07-17 09:59:07 +0200 (jeu, 17 jui 2008) $
 *  Id:      $Id: Log.hxx 884 2008-07-17 07:59:07Z dutka $
 */
#ifndef OPENTURNS_LOG_HXX
#define OPENTURNS_LOG_HXX

#include <list>
#include <fstream>
#include "OT.hxx"
#include "OTconfig.hxx"

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
// #else
// #error "Header file pthread.h is MANDATORY"
#endif

#ifdef HAVE_SEMAPHORE_H
#include <semaphore.h>
// #else
// #error "Header file semaphore.h is MANDATORY"
#endif

#include "Lockable.hxx"
#include "Threadable.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Common
    {


      /**
       * Log records all user information to a file or tty
       */

      class Log :
	public Lockable,
	public Threadable
      {
      public:

	enum Severity { DBG, INFO, WARN, ERROR };

	// Log messages according to its relative severity
	static void Debug(const String & msg);
	static void Info(const String & msg);
	static void Warn(const String & msg);
	static void Error(const String & msg);

	static void SetMinSeverity(Severity sev);

	static Log & GetInstance();
	static void Release();
	static void * Run(void * data);

      private:
	static Log * P_instance_;
	static pthread_mutex_t InstanceMutex_;
	static unsigned long Index_;


      public:
	
	struct Entry
	{
	  Severity sev_;
	  String msg_;
	  Entry(Severity sev, String msg) : sev_(sev), msg_(msg) {}
	}; /* end struct Entry */

	typedef Lockable::Lock Lock;

	~Log();

	/** Set the name of the log file */
	void setFile(const FileName & file);

	/** Regularly flush the log to the file */
	void run();

	// Get the current number of entry in the log
	// For test purpose only
	UnsignedLong getSize() const;

      private:
	Log();
	void setSeverity(Severity sev)const ;
	Bool shouldWork() const;
	void flush();
	void print();
	void stopWorking();
	void push(const Entry & entry);

	/** The list of messages to be printed */
	std::list<Entry> messages_;

	/** The file where to write messages */
	std::ostream * p_file_;

	/** A semaphore to alert when messages are in list */
	//mutable sem_t sem_messages_;

	/** The condition that informs something has changed on message list */
	//mutable pthread_cond_t list_cond_;

	/** Controls if thread is working */
	mutable Bool working_;

	/** Controls if pushing new data onto the list is allowed */
	mutable Bool pushAllowed_;

	/** Tell if flushing has been done */
	mutable Bool flushed_;

	/** Thread Id */
	//pthread_t tid_;

	/* Minimum severity logged to the file */
	mutable Severity minSeverity_;

      }; /* end class Log */






    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_LOG_HXX */
