//                                               -*- C++ -*-
/**
 *  @file  Matrix.hxx
 *  @brief Matrix implements the classical mathematical matrix
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-05-02 14:59:25 +0200 (mer, 02 mai 2007) $
 *  Id:      $Id: Matrix.hxx 424 2007-05-02 12:59:25Z dutka $
 */
#ifndef OPENTURNS_MATRIX_HXX
#define OPENTURNS_MATRIX_HXX

#include "TypedInterfaceObject.hxx"
#include "MatrixImplementation.hxx"




namespace OpenTURNS
{

  namespace Base
  {
  
    namespace Type
    {
    
      class IdentityMatrix;

      class SymmetricMatrix;

      /**
       * @class Matrix
       *
       * Matrix implements the classical mathematical matrix
       */ 

      class Matrix :
	public Common::TypedInterfaceObject<MatrixImplementation>
      {
	CLASSNAME;
	
#ifndef SWIG
	/** Declaration of friend operators */
	friend Matrix operator * (const NumericalScalar & s, const Matrix & m);
#endif
	
	
      public:
	
	typedef Common::InvalidDimensionException InvalidDimensionException ;
	typedef Common::InvalidArgumentException InvalidArgumentException ;
	typedef Common::TypedInterfaceObject<MatrixImplementation>::Implementation Implementation ;
	
	/** Default constructor */
	Matrix(const String & name = OT::DefaultName );
	
	/** Constructor with size (rowDim and colDim) */
	Matrix(UnsignedLong rowDim, UnsignedLong colDim, const String & name = OT::DefaultName);

	/** Constructor from range of external collection */
	template <class InputIterator>
	Matrix(UnsignedLong rowDim, UnsignedLong colDim,
	       InputIterator first, InputIterator last,
	       const String & name = OT::DefaultName);
       
	/** Constructor from external collection */
	/** If the dimensions of the matrix and of the collection */
	/** do not correspond, either the collection is truncated */
	/** or the rest of the matrix is filled with zeros */
	Matrix(UnsignedLong rowDim, UnsignedLong colDim,
	       const Collection<NumericalScalar> & elementsValues,
	       const String & name = OT::DefaultName);
	

	/** String converter */
	virtual String str() const;

#ifndef SWIG
	/** Operator () gives access to the elements of the matrix (to modify these elements) */
	/** The element of the matrix is designated by its row number i and its column number j */
	NumericalScalar & operator () (UnsignedLong i, UnsignedLong j) throw(InvalidDimensionException);
	
	/** Operator () gives access to the elements of the matrix (read only) */
	/** The element of the matrix is designated by its row number i and its column number j */
	const NumericalScalar & operator () (UnsignedLong i, UnsignedLong j) const throw(InvalidDimensionException);
#endif

	/** Get the dimensions of the matrix */
	/** Number of rows */
	const UnsignedLong getNbRows() const ;
	/** Number of columns */
	const UnsignedLong getNbColumns() const ;
	
	/** Matrix transpose */
	Matrix transpose () const ;
	
	/** Matrix additions (must have the same dimensions) */
	Matrix operator + (const Matrix & m) const throw(InvalidDimensionException);
	Matrix operator + (const SymmetricMatrix & m) const throw(InvalidDimensionException);
	
	/** Matrix substractions (must have the same dimensions) */
	Matrix operator - (const Matrix & m) const throw(InvalidDimensionException);
	Matrix operator - (const SymmetricMatrix & m) const throw(InvalidDimensionException);
	
	/** Matrix multiplications (must have consistent dimensions) */
	Matrix operator * (const Matrix & m) const throw(InvalidDimensionException);
	Matrix operator * (const SymmetricMatrix & m) const throw(InvalidDimensionException);
	Matrix operator * (const IdentityMatrix & m) const throw(InvalidDimensionException);
	
	/** Multiplication with a NumericalPoint (must have consistent dimensions) */
	NumericalPoint operator * (const NumericalPoint & pt) const throw(InvalidDimensionException);

	/** Multiplication with a NumericalScalar */
	Matrix operator * (const NumericalScalar & s) const ;
	
	/** Division by a NumericalScalar*/
	Matrix operator / (const NumericalScalar & s) const throw(InvalidArgumentException);
	
	/** Resolution of a linear system */
	NumericalPoint solveLinearSystem(const NumericalPoint & b) const throw(InvalidDimensionException);
	
	/** Comparison operators */
	Bool operator == (const Matrix & rhs) const ;
	
	/** Empty returns true if there is no element in the matrix */
	const Bool isEmpty() const;
	
	/** Constructor with implementation */
	Matrix(const Implementation & i);

      }; /* class Matrix */
	
      /** Declaration of friend operators */
      inline Matrix operator * (const NumericalScalar & s, const Matrix & m)
      {
	return m.operator*(s);
      }


      /** Constructor from range of external collection */
      template <class InputIterator>
      Matrix::Matrix(UnsignedLong rowDim, // Number of rows of the matrix
                     UnsignedLong colDim, // Number of columns of the matrix
	             InputIterator first, // First element of the external array
		     InputIterator last, // Last element of the external array
	             const String & name // The name the user gave to the object
		     )
        : Common::TypedInterfaceObject<MatrixImplementation>(new MatrixImplementation(rowDim,colDim,first,last,name))
      {
        // Nothing to do
      }

    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_MATRIX_HXX */
