//                                               -*- C++ -*-
/**
 * @file  NumericalMathEvaluationImplementation.hxx
 * @brief Abstract top-level class for all numerical math function implementations
 *
 * (C) Copyright 2005-2006 EDF
 *
 * Permission to copy, use, modify, sell and distribute this software
 * is granted provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied
 * warranty, and with no claim as to its suitability for any purpose.
 *
 *
 * \author $LastChangedBy: dutka $
 * \date   $LastChangedDate: 2008-06-26 13:50:17 +0200 (jeu, 26 jun 2008) $
 */

#ifndef OPENTURNS_NONUMERICALMATHFUNCTIONIMPLEMENTATION_HXX
#define OPENTURNS_NONUMERICALMATHFUNCTIONIMPLEMENTATION_HXX

#include "NumericalMathEvaluationImplementation.hxx"

namespace OpenTURNS {

  namespace Base {

    namespace Func {


      /**
       * @class NoNumericalMathEvaluationImplementation
       *
       * This class offers an interface for an real numerical mathematical function
       * implementations into the platform. It realizes the computation of the image vector
       * of a vector (aka NumericalPoint) through the function. The
       * function may be loaded in an external way through a wrapper.
       */
      class NoNumericalMathEvaluationImplementation
	: public NumericalMathEvaluationImplementation
      {
	CLASSNAME;
      public:

	typedef NumericalMathEvaluationImplementation::Description Description;
	typedef NumericalMathEvaluationImplementation::NumericalPoint NumericalPoint;
	typedef NumericalMathEvaluationImplementation::Matrix Matrix;
	typedef NumericalMathEvaluationImplementation::InvalidArgumentException InvalidArgumentException;
	typedef NumericalMathEvaluationImplementation::InternalException InternalException;
	typedef NumericalMathEvaluationImplementation::NumericalSample NumericalSample;


	/** Default constructor */
	NoNumericalMathEvaluationImplementation(const String & name = OT::DefaultName);

	/** Virtual constructor */
	virtual NoNumericalMathEvaluationImplementation * clone() const;

	/** Comparison operator */
	Bool operator ==(const NoNumericalMathEvaluationImplementation & other) const;

	/** String converter */
	virtual String str() const;


	/** Test for actual implementation */
	virtual Bool isActualImplementation() const;


	/* Here is the interface that all derived class must implement */
	
	/** Operator () */
	virtual NumericalPoint operator() (const NumericalPoint & in) const
	  throw(InvalidArgumentException,InternalException);

	/** Accessor for input point dimension */
	virtual UnsignedLong getInputNumericalPointDimension() const
	  throw(InternalException);

	/** Accessor for output point dimension */
	virtual UnsignedLong getOutputNumericalPointDimension() const
	  throw(InternalException);

 	/** Method save() stores the object through the StorageManager
	 *  The inherited method is sufficient as we do not have any
         *  attribute
         */

	/** Method load() reloads the object from the StorageManager
	 *  The inherited method is sufficient as we do not have any
         *  attribute
         */

      protected:

    
      private:

      }; /* class NoNumericalMathEvaluationImplementation */


    } /* namespace Func */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_NONUMERICALMATHFUNCTIONIMPLEMENTATION_HXX */
