//                                               -*- C++ -*-
/**
 *  @file  NonCentralStudent.hxx
 *  @brief The NonCentralStudent distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-06-26 13:50:17 +0200 (jeu, 26 jun 2008) $
 *  Id:      $Id: NonCentralStudent.hxx 862 2008-06-26 11:50:17Z dutka $
 */
#ifndef OPENTURNS_NONCENTRALSTUDENT_HXX
#define OPENTURNS_NONCENTRALSTUDENT_HXX

#include "NonEllipticalDistribution.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {


      /**
       * @class NonCentralStudent
       *
       * The NonCentralStudent distribution.
       */
      class NonCentralStudent 
	: public Model::NonEllipticalDistribution
      {
	CLASSNAME;
      public:

	typedef Base::Common::InvalidArgumentException               InvalidArgumentException;
	typedef Model::NonEllipticalDistribution                     NonEllipticalDistribution;  // required by SWIG
	typedef NonEllipticalDistribution::NumericalPoint            NumericalPoint;
	typedef NonEllipticalDistribution::NumericalPointWithDescriptionCollection  NumericalPointWithDescriptionCollection;
	typedef NonEllipticalDistribution::NotDefinedException       NotDefinedException;
	typedef NonEllipticalDistribution::StorageManager            StorageManager;


	/** Default constructor */
	NonCentralStudent(const NumericalScalar nu = 5.0, const NumericalScalar delta = 0.0, const NumericalScalar gamma = 0.0)
	  throw(InvalidArgumentException);

	/** Comparison operator */
	Bool operator ==(const NonCentralStudent & other) const;

	/** String converter */
	String str() const;



	/* Interface inherited from Distribution */

	/** Virtual constructor */
	NonCentralStudent * clone() const;
	
	/** Get one realization of the distribution */
	NumericalPoint getRealization() const;

	/** Get the PDF of the distribution */
	using NonEllipticalDistribution::computePDF;
	NumericalScalar computePDF(const NumericalPoint & point) const;

	/** Get the CDF of the distribution */
	using NonEllipticalDistribution::computeCDF;
	NumericalScalar computeCDF(const NumericalPoint & point) const;

	/** Get the PDFGradient of the distribution */
	NumericalPoint computePDFGradient(const NumericalPoint & point) const;

	/** Get the CDFGradient of the distribution */
	NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

	/** Get the mean of the distribution */
	NumericalPoint getMean() const throw(NotDefinedException);

	/** Get the standard deviation of the distribution */
	NumericalPoint getStandardDeviation() const throw(NotDefinedException);

	/** Get the skewness of the distribution */
	NumericalPoint getSkewness() const throw(NotDefinedException);

	/** Get the kurtosis of the distribution */
	NumericalPoint getKurtosis() const throw(NotDefinedException);

	/** Get the covariance of the distribution */
	CovarianceMatrix getCovariance() const throw(NotDefinedException);

	/** Parameters value and description accessor */
	NumericalPointWithDescriptionCollection getParametersCollection() const;

	/** Nu accessor */
	void setNu(const NumericalScalar nu)
	  throw(InvalidArgumentException);
	NumericalScalar getNu() const;

	/** Delta accessor */
	void setDelta(const NumericalScalar delta);
	NumericalScalar getDelta() const;

	/** Gamma accessor */
	void setGamma(const NumericalScalar gamma);
	NumericalScalar getGamma() const;

	/** Method save() stores the object through the StorageManager */
	void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	void load(const StorageManager::Advocate & adv);


      protected:
    
    
      private:
	// Number of degrees of freedom
	NumericalScalar nu_;
	// Non-centrality parameter
	NumericalScalar delta_;
	// Location parameter
	NumericalScalar gamma_;

      }; /* class NonCentralStudent */


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_NONCENTRALSTUDENT_HXX */
