//                                               -*- C++ -*-
/**
 *  @file  Normal.hxx
 *  @brief The Normal distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-05-21 11:21:38 +0200 (mer, 21 mai 2008) $
 *  Id:      $Id: Normal.hxx 815 2008-05-21 09:21:38Z dutka $
 */
#ifndef OPENTURNS_NORMAL_HXX
#define OPENTURNS_NORMAL_HXX

#include "OT.hxx"
#include "EllipticalDistribution.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      /**
       * @class Normal
       *
       * The Normal (gaussian) distribution.
       */
      class Normal 
	: public Model::EllipticalDistribution
      {
	static const UnsignedLong MaximumNumberOfPoints = 10000000;
	static const UnsignedLong MinimumNumberOfPoints = 1000000;
	static const NumericalScalar MaximumCDFEpsilon = 5.0e-6;
	static const NumericalScalar MinimumCDFEpsilon = 5.0e-2;

	CLASSNAME;
      public:

	typedef Model::EllipticalDistribution                                 EllipticalDistribution; // required by SWIG
	typedef EllipticalDistribution::Implementation                        Implementation;
	typedef EllipticalDistribution::InvalidArgumentException              InvalidArgumentException;
	typedef EllipticalDistribution::InvalidDimensionException             InvalidDimensionException;
	typedef EllipticalDistribution::NotDefinedException                   NotDefinedException;
	typedef EllipticalDistribution::NumericalPoint                        NumericalPoint;
	typedef EllipticalDistribution::NumericalSample                       NumericalSample;
	typedef EllipticalDistribution::CovarianceMatrix                      CovarianceMatrix;
	typedef EllipticalDistribution::Indices                               Indices;
	typedef EllipticalDistribution::Interval                              Interval;
	typedef EllipticalDistribution::IsoProbabilisticTransformation        IsoProbabilisticTransformation;
	typedef EllipticalDistribution::InverseIsoProbabilisticTransformation InverseIsoProbabilisticTransformation;
	typedef EllipticalDistribution::CorrelationMatrix                     CorrelationMatrix;
	typedef EllipticalDistribution::SymmetricMatrix                       SymmetricMatrix;
	typedef EllipticalDistribution::SquareMatrix                          SquareMatrix;
	typedef EllipticalDistribution::StorageManager                        StorageManager;
	
	/** Default constructor */
	explicit Normal(const UnsignedLong dimension = 1)
	  throw(InvalidArgumentException);

	/** Dimension 1 constructor */
	explicit Normal(const NumericalScalar mu, const NumericalScalar sd)
	  throw(InvalidArgumentException);

	/** Constructor for multiD normal distribution */
	explicit Normal(const NumericalPoint & mean,
			const NumericalPoint & sigma,
			const CorrelationMatrix & R)
	  throw(InvalidArgumentException);

	explicit Normal(const NumericalPoint & mean,
			const CovarianceMatrix & C)
	  throw(InvalidArgumentException);


	/** String converter */
	String str() const;

	/* Interface inherited from Distribution */

	/** Virtual constructor */
	Normal * clone() const;

	/** Compute the density generator of the ellipticalal generator, i.e.
	 *  the function phi such that the density of the distribution can
	 *  be written as p(x) = phi(t(x-mu)R^(-1)(x-mu))                      */
	NumericalScalar computeDensityGenerator(const NumericalScalar betaSquare) const;

	/** Compute the derivative of the density generator */
	NumericalScalar computeDensityGeneratorDerivative(const NumericalScalar betaSquare) const;

	/** Compute the seconde derivative of the density generator */
	NumericalScalar computeDensityGeneratorSecondDerivative(const NumericalScalar betaSquare) const;

	/** Get one realization of the Normal distribution */
	NumericalPoint getRealization() const;

	/** Get the CDF of the Normal distribution */
	using EllipticalDistribution::computeCDF;
	NumericalScalar computeCDF(const NumericalPoint & point) const;

	/** Get the probability content of an interval */
	NumericalScalar computeProbability(const Interval & interval) const;

	/** Get the characteristic function of the distribution, i.e. phi(u) = E(exp(I*u*X)) */
	NumericalComplex computeCharacteristicFunction(const NumericalPoint & point) const;

	/** Get the CDF gradient of the distribution */
	NumericalPoint computeCDFGradient(const NumericalPoint & point) const;

	/** Compute the radial distribution CDF */
	NumericalScalar computeRadialDistributionCDF(const NumericalScalar radius) const;

	/** Get the i-th marginal distribution */
	Implementation getMarginal(const UnsignedLong i) const throw(InvalidArgumentException);

	/** Get the distribution of the marginal distribution corresponding to indices dimensions */
	Implementation getMarginal(const Indices & indices) const throw(InvalidArgumentException);

	/** Get the roughness, i.e. the L2-norm of the PDF */
	NumericalScalar getRoughness() const;

	/** Get the skewness of the distribution */
	NumericalPoint getSkewness() const throw(NotDefinedException);

	/** Get the kurtosis of the distribution */
	NumericalPoint getKurtosis() const throw(NotDefinedException);

	/** Sigma vector accessor */
	void setSigma(const NumericalPoint & sigma)
	  throw(InvalidArgumentException);

	/** Correlation matrix accessor */
	void setCorrelationMatrix(const CorrelationMatrix & R)
	  throw(InvalidArgumentException);

	/** Get the standard distribution, i.e. a distribution of the same kind but with zero mean,
	 * unit marginal standard distribution and identity correlation */
	Implementation getStandardDistribution() const;

	/** Get the copula of a distribution */
	Implementation getCopula() const;

	/* Interface specific to Normal */

	/** Tell if the distribution has independent copula */
	Bool hasIndependentCopula() const;

	/** Method save() stores the object through the StorageManager */
	void save(const StorageManager::Advocate & adv) const;

	/** Method load() reloads the object from the StorageManager */
	void load(const StorageManager::Advocate & adv);

      protected:
    

      private:

	/** Quantile computation for dimension=1 */
	NumericalScalar computeScalarQuantile(const NumericalScalar prob, const NumericalScalar initialGuess = 0.0, const NumericalScalar initialStep = 1.0) const;
    
        /** Check if the distribution has independent copula */
        void checkIndependentCopula();

	/** The normalization factor of the Normal distribution */
	NumericalScalar normalizationFactor_;

	/** Store the independence status */
	Bool hasIndependentCopula_;

      }; /* class Normal */

    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_NORMAL_HXX */
