//                                               -*- C++ -*-
/**
 *  @file  Tensor.hxx
 *  @brief Tensor implements the classical mathematical tensor
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-05-02 14:59:25 +0200 (mer, 02 mai 2007) $
 *  Id:      $Id: Tensor.hxx 424 2007-05-02 12:59:25Z dutka $
 */
#ifndef OPENTURNS_TENSOR_HXX
#define OPENTURNS_TENSOR_HXX

#include "TypedInterfaceObject.hxx"
#include "TensorImplementation.hxx"


namespace OpenTURNS
{

  namespace Base
  {

    namespace Type
    {

      /**
       * @class Tensor
       *
       * Tensor implements the classical mathematical Tensor
       */

      class Tensor :
	public Common::TypedInterfaceObject<TensorImplementation>
      
      {
	CLASSNAME;
	
      public:
	
	typedef Common::InvalidDimensionException InvalidDimensionException ;
	typedef Common::InvalidArgumentException InvalidArgumentException ;
	typedef Common::TypedInterfaceObject<TensorImplementation>::Implementation Implementation ;
	
	/** Default constructor */
	Tensor(const String & name = OT::DefaultName);

	/** Constructor with size (rowDim, colDim, sheetDim) */
	Tensor(UnsignedLong rowDim, UnsignedLong colDim, UnsignedLong sheetDim, const String & name = OT::DefaultName);
	
	/** Constructor from range of external collection */
	template <class InputIterator>
	Tensor(UnsignedLong rowDim, UnsignedLong colDim, UnsignedLong sheetDim,
	       InputIterator first, InputIterator last,
	       const String & name = OT::DefaultName);
	       
	/** Constructor from external collection */
	Tensor(UnsignedLong rowDim, UnsignedLong colDim, UnsignedLong sheetDim, const Collection<NumericalScalar> & elementsValues, const String & name = OT::DefaultName);
	
	/** String converter */
	virtual String str() const;

#ifndef SWIG	
	/** Operator () gives access to the elements of the tensor (to modify these elements) */
	/** The element of the tensor is designated by its row number i, its column number j and its sheet number k */
	NumericalScalar & operator () (UnsignedLong i, UnsignedLong j, UnsignedLong k) throw(InvalidDimensionException);
	
	/** Operator () gives access to the elements of the tensor (read only) */
	/** The element of the tensor is designated by its row number i, its column number j and its sheet number k */
	const NumericalScalar & operator () (UnsignedLong i, UnsignedLong j, UnsignedLong k) const throw(InvalidDimensionException);
#endif

	/** getSheet returns the sheet specified by its sheet number k */
	Matrix getSheet(UnsignedLong k) const throw(InvalidDimensionException);
	
	/** setSheet sets matrix m as the sheet specified by its sheet number k  */
	void setSheet(UnsignedLong k, Matrix & m) throw(InvalidDimensionException);
	
	/** Get the dimensions of the tensor */
	/** Number of rows */
	const UnsignedLong getNbRows() const ;
	/** Number of columns */
	const UnsignedLong getNbColumns() const ;
	/** Number of sheets */
	const UnsignedLong getNbSheets() const ;
	
	/** Comparison operators */
	Bool operator == (const Tensor & rhs) const ;
	
	/** Empty returns true if there is no element in the tensor */
	const Bool isEmpty() const;

      protected:
        
	/** Constructor with implementation */
	Tensor(const Implementation & i);

      }; /* class Tensor */



      /** Constructor with size (rowDim, colDim, sheetDim) */
      template <class InputIterator>
      Tensor::Tensor(UnsignedLong rowDim, UnsignedLong colDim, UnsignedLong sheetDim,
		     InputIterator first, InputIterator last,
		     const String & name)
	: Common::TypedInterfaceObject<TensorImplementation>(new TensorImplementation(rowDim,colDim,sheetDim,first,last,name))
      {
	// nothing to do
      }

    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_MATRIX_HXX */
