//                                               -*- C++ -*-
/**
 *  @file  WrapperData.hxx
 *  @brief This class declares the wrapper data that are exchanged with the platform
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-08-01 16:47:51 +0200 (mer, 01 aoû 2007) $
 *  Id:      $Id: WrapperData.hxx 488 2007-08-01 14:47:51Z dutka $
 */
#ifndef OPENTURNS_WRAPPERDATA_HXX
#define OPENTURNS_WRAPPERDATA_HXX

#include <iostream>              // for std::ostream
#include <vector>                // for std::vector
#include "OT.hxx"
#include "Pointer.hxx"           // for shared pointer
#include "WrapperInterface.h"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Common
    {

      /**
       * @class WrapperSymbolProvided
       *
       * Stores information about the presence or no of symbols in the wrapper
       */
      class WrapperSymbolProvided {
      public:
	typedef Bool Value;
	static const Value NO  = WRAPPER_NO;
	static const Value YES = WRAPPER_YES;
      };


      /**
       * @class WrapperComputedGradient
       *
       * Stores information about the computation of gradient for a variable
       */
      class WrapperComputedGradient {
      public:
	typedef Bool Value;
	static const Value NO  = WRAPPER_NO;
	static const Value YES = WRAPPER_YES;
      };


      /**
       * @class WrapperDataFileType
       *
       * Stores the type (input or output) of the file
       */
      class WrapperDataFileType {
      public:
	typedef unsigned long Value;
	static const Value IN  = WRAPPER_IN;
	static const Value OUT = WRAPPER_OUT;
      };

      /**
       * @struct WrapperDataFile
       *
       * This struct holds the name and the path of any file passed as argument to the wrapper
       */
      struct WrapperDataFile {
	String id_;                       ///< The id of the file (any string to distinguish each file from another)
	String name_;                     ///< The name of the file (stdin, stdout, log, etc.)
	FileName path_;                   ///< The path of the file (/tmp/stdin, /var/log/mylog, etc.)
	WrapperDataFileType::Value type_; ///< The type of the file (input or output)
      }; /* struct WrapperDataFile */




      /**
       * @class WrapperDataVariableType
       *
       * Stores the type (input or output) of the variable
       */
      class WrapperDataVariableType {
      public:
	typedef unsigned long Value;
	static const Value IN  = WRAPPER_IN;
	static const Value OUT = WRAPPER_OUT;
      };

      /**
       * @struct WrapperDataVariable
       *
       * This struct holds information of any variable passed as argument to the wrapper
       */
      struct WrapperDataVariable {
	String id_;                               ///< The id of the variable (any string to distinguish each variable from another)
	String comment_;                          ///< A comment to inform on the variable
	String unit_;                             ///< The unit which the variable is expressed in
	String regexp_;                           ///< The regular expression used to find the variable location in files
	String format_;                           ///< The format in which the variable must be printed in files
	WrapperDataVariableType::Value type_;     ///< The type of the variable (input or output)
	WrapperComputedGradient::Value gradient_; ///< The gradient of this variable is computed if true
      }; /* struct WrapperDataVariable */




      /**
       * @class WrapperState
       *
       * Stores the mode of invocation of the external code
       */
      class WrapperState {
      public:
	typedef unsigned long Value;
	static const Value SHARED   = WRAPPER_SHAREDSTATE;
	static const Value SPECIFIC = WRAPPER_SPECIFICSTATE;
      };


      /**
       * @class WrapperMode
       *
       * Stores the mode of invocation of the external code
       */
      class WrapperMode {
      public:
	typedef unsigned long Value;
	static const Value STATICLINK  = WRAPPER_STATICLINK;
	static const Value DYNAMICLINK = WRAPPER_DYNAMICLINK;
	static const Value FORK        = WRAPPER_FORK;
      };


      /**
       * @class WrapperDataTransfer
       *
       * Stores the mode of transmission for the arguments
       */
      class WrapperDataTransfer {
      public:
	typedef unsigned long Value;
	static const Value FILES     = WRAPPER_FILES;
	static const Value PIPE      = WRAPPER_PIPE;
	static const Value ARGUMENTS = WRAPPER_ARGUMENTS;
	static const Value SOCKET    = WRAPPER_SOCKET;
	static const Value CORBA     = WRAPPER_CORBA;
      };


      /**
       * struct WrapperParameter
       *
       * This structure holds the configuration of the wrapper
       */
      struct WrapperParameter {
	WrapperState::Value        state_; ///< The sharing mode of internal state
	WrapperMode::Value         mode_;  ///< The mode of invocation of the external code
	WrapperDataTransfer::Value in_;    ///< The transmission mode for input arguments
	WrapperDataTransfer::Value out_;   ///< The transmission mode for output arguments
	String command_;                   ///< The command that should invoque the external code according to mode
      }; /* struct WrapperParameter */



      /**
       * @class WrapperData
       *
       * This class declares the wrapper data that are exchanged with the platform
       */
      class WrapperData {
      public:
	typedef std::vector<WrapperDataFile>     FileListType;
	typedef std::vector<WrapperDataVariable> VariableListType;

	/** Default constructor */
	explicit WrapperData();

	/** Constructor */
	WrapperData(const FileListType & fileList, const VariableListType & variableList, const WrapperParameter & parameters);

	/** Destructor */
	virtual ~WrapperData();

	/** Accessor */
	void setFileList(const FileListType & fileList);
	const FileListType & getFileList() const;

	/** Conversion method for C interface */
	struct WrapperFileList * getNewFileListForCInterface() const;

	/** Frees the memory allocated by getNewFileListForCInterface() method */
	void freeFileListForCInterface(const struct WrapperFileList * fileList) const;

	/** Accessor */
	void setVariableList(const VariableListType & variableList);
	const VariableListType & getVariableList() const;

	/** Conversion method for C interface */
	struct WrapperVariableList * getNewVariableListForCInterface() const;

	/** Frees the memory allocated by getNewVariableListForCInterface() method */
	void freeVariableListForCInterface(const struct WrapperVariableList * variableList) const;

	/** Accessor */
	void setParameters(const WrapperParameter & parameters);
	const WrapperParameter & getParameters() const;

	/** Conversion method for C interface */
	struct WrapperConfiguration * getNewParametersForCInterface() const;

	/** Frees the memory allocated by getNewParametersForCInterface() method */
	void freeParametersForCInterface(const struct WrapperConfiguration * parameters) const;

      protected:

      private:

	/** A shared pointer on the list of files that are exchanged between the platform and the wrapper */
	Pointer<FileListType> p_fileList_;

	/** A shared pointer on the list of variables that are exchanged between the platform and the wrapper */
	Pointer<VariableListType> p_variableList_;

	/** A shared pointer on the parameters for the wrapper */
	Pointer<WrapperParameter> p_parameters_;

      }; /* class WrapperData */


    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_WRAPPERDATA_HXX */
