//                                               -*- C++ -*-
/**
 *  @file  WrapperFile.hxx
 *  @brief This class provides all the treatments for wrapper file manipulation
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-05-21 17:44:02 +0200 (mer, 21 mai 2008) $
 *  Id:      $Id: WrapperFile.hxx 818 2008-05-21 15:44:02Z dutka $
 */
#ifndef OPENTURNS_WRAPPERFILE_HXX
#define OPENTURNS_WRAPPERFILE_HXX

#include <iostream>              // for std::ostream
#include "OT.hxx"
#include "WrapperData.hxx"
#include "Exception.hxx"

namespace OpenTURNS
{

  namespace Base
  {

    namespace Common
    {

      /**
       * @class WrapperFile
       *
       * This class provides all the treatments for wrapper file manipulation
       */

      class WrapperFile
      {
      public:

	/** Struct FunctionDescription describes what is the interface af a numerical function */
	struct FunctionDescription {
	  String name_;
	  Bool   provided_;
	};

      public:

	/**
	 * Find the path of a wrapper file from its name.
	 */
	static FileName FindWrapperPathByName(const String & name)
	  throw(NoWrapperFileFoundException);

      protected:

	/** The file name extension */
	const static String extension_;
	
	/** The environment variable name */
	const static String openturnsWrapperPathVariableName_;
	
	/** The HOME subdirectory path */
	const static String homeSubdirectory_;
	
	/** The 'prefix' subdirectory path */
	const static String prefixSubdirectory_;




      public:

	/** 
	 * Default constructor
	 */
	explicit WrapperFile(const FileName & pathToFile)
	  throw(WrapperFileParsingException);

	/** Destructor */
	virtual ~WrapperFile();

	/** Description file path accessor */
	FileName getDescriptionFilePath() const;

	/** Library path accessor */
	FileName getLibraryPath() const;

	/** Function description accessor */
	FunctionDescription getFunctionDescription() const;

	/** Gradient description accessor */
	FunctionDescription getGradientDescription() const;

	/** Hessian description accessor */
	FunctionDescription getHessianDescription() const;

	/** Wrapper data accessor */
	WrapperData getWrapperData() const;


      protected:

	/** Where the description file of the wrapper is located */
	FileName descriptionFilePath_;

	/** Where the library that hold the function is located */
	FileName libraryPath_;

	/** The description of the function to be bound to */
	FunctionDescription function_;

	/** The description of the gradient of the function */
	FunctionDescription gradient_;

	/** The description of the hessian of the function */
	FunctionDescription hessian_;

	/** The data that the wrapper may exchange with the platform */
	WrapperData data_;

      private:

	/** Copy constructor */
	WrapperFile(const WrapperFile & other)
	  throw(WrapperFileParsingException);

	/** Assignment */
	WrapperFile & operator =(const WrapperFile & other);

	/** Initialization */
	void init() const
	  throw(WrapperFileParsingException);

	/** Finalization */
	void done() const;

	/** Parsing function */
	void parseFile(const FileName & pathToFile)
	  throw(WrapperFileParsingException);

      }

      ; /* class WrapperFile */


    } /* namespace Common */
  } /* namespace Base */
} /* namespace OpenTURNS */

#endif /* OPENTURNS_WRAPPERFILE_HXX */
